export default function (server) {

  let _ = require('lodash');
  let call = server.plugins.elasticsearch.callWithRequest || server.plugins.elasticsearch.getCluster('data').callWithRequest;

  let getAggs = function(fieldNames) {
    let result = [];
    for (let i = 0, len = fieldNames.length; i < len; i++) {
      result.push({
        name: fieldNames[i],
        order: {
          _term: 'desc'
        },
        size: 10000
      });
    }
    return result;
  };

  /**
   * @api {get} /api/kibana_iap_portlet_plugin/distinct_items 項目名を取得します
   * @apiDescription
   * fieldnames に指定したフィールド名に含まれる項目名を検索します。
   *
   * 検索は indexname に指定したインデックスに対して range の範囲で行います。
   * range が指定されなかった場合、全データが対象です。
   *
   * @apiParam {String[]} fieldnames 項目名を取得したいフィールド名の配列
   * @apiParam {String} indexname インデックス名
   * @apiParam {Object} range 検索範囲
   *
   * @apiSuccess {Object} result 検索結果
   * @apiSuccess {String[]} result[fieldName] フィールド名に含まれる項目名
   */
  server.route({
    path: '/api/kibana_iap_portlet_plugin/distinct_items',
    method: ['GET'],
    handler(req, reply) {
      let fieldNames = [];
      if (req.query && req.query.fieldnames) {
        fieldNames = JSON.parse(req.query.fieldnames);
      }
      let aggs = getAggs(fieldNames);
      let range = req.query.range;
      let indexName = req.query.indexname;
      if (!indexName) {
        server.log(['error', 'kibana-iap-portlet-plugin', 'distinct_items'], 'Index name is not specified.');
        reply({});
      }

      let searchParams = {
        index: indexName,
        body: {
          "size":0,
          "aggs":{
          }
        }
      }
      if (range) {
        searchParams.body.aggs.distinct_items.filter = range;
      }
      for (var i = 0, len = aggs.length; i < len; i++) {
        var agg = aggs[i];
        searchParams.body.aggs[agg.name] = {
          terms: {
            field: agg.name,
            order: agg.order,
            size: agg.size
          }
        }
      }
      server.log(['debug', 'kibana-iap-portlet-plugin', 'distinct_items'], JSON.stringify(searchParams));

      call(req, 'search', searchParams).then(function(response) {
        let names = _.map(aggs, 'name');
        let result = {};
        for (let i = 0, len = names.length; i < len; i++) {
          result[names[i]] = _.map(response.aggregations[names[i]].buckets, 'key');
        }
        reply(result);
      });
    }
  });
};


